<?php

declare(strict_types = 1);

namespace Drupal\zalo_zns\Plugin\rest\resource;

use Drupal\message\Entity\Message;
use Drupal\rest\ModifiedResourceResponse;
use Drupal\rest\Plugin\ResourceBase;
use Symfony\Component\HttpFoundation\Request;

/**
 * Represents Zalo Notification Webhook records as resources.
 *
 * @RestResource (
 *   id = "zalo_zns_zalo_notification_webhook",
 *   label = @Translation("Zalo Notification Webhook"),
 *   uri_paths = {
 *     "create" = "/api/zalo-zns-zalo-notification-webhook"
 *   }
 * )
 *
 * @DCG
 * The plugin exposes key-value records as REST resources. In order to enable it
 * import the resource configuration into active configuration storage. An
 * example of such configuration can be located in the following file:
 * core/modules/rest/config/optional/rest.resource.entity.node.yml.
 * Alternatively, you can enable it through admin interface provider by REST UI
 * module.
 * @see https://www.drupal.org/project/restui
 *
 * @DCG
 * Notice that this plugin does not provide any validation for the data.
 * Consider creating custom normalizer to validate and normalize the incoming
 * data. It can be enabled in the plugin definition as follows.
 * @code
 *   serialization_class = "Drupal\foo\MyDataStructure",
 * @endcode
 *
 * @DCG
 * For entities, it is recommended to use REST resource plugin provided by
 * Drupal core.
 * @see \Drupal\rest\Plugin\rest\resource\EntityResource
 */
final class ZaloNotificationWebhookResource extends ResourceBase {

  /**
   * Responds to POST requests and saves the new record.
   */
  public function post(Request $request, array $data): ModifiedResourceResponse {
    $server = $request->headers->get('X-Zevent-Server');
    $signature = $request->headers->get('X-Zevent-Signature');

    if ($server == 'ZNS') {
      $appId = (string) \Drupal::config('zalo_zns.settings')->get('app_id');
      $oaSecretKey = \Drupal::config('zalo_zns.settings')->get('oa_secret_key');
      $jsonEncode = json_encode($data);
      $verifySignature = 'mac=' . hash('sha256', $appId . $jsonEncode . $data['timestamp'] . $oaSecretKey);
      if ($verifySignature == $signature) {
        if ($data['event_name'] == 'user_received_message') {
          $message = Message::load($data['message']['tracking_id']);
          if ($message) {
            $message->set('field_status', 'success')->save();
            \Drupal::logger('zalo_zns')->info("User received zns {$message->id()} successfully");
          }
        }
      }
      else {
        \Drupal::logger('zalo_zns')->error("X-Zevent-Signature is not match");
      }
    }

    return new ModifiedResourceResponse('OK', 200);
  }

}
